Author: Armin Rigo <[email protected]>
Branch:
Changeset: r52086:382248c1c015
Date: 2012-02-04 14:17 +0100
http://bitbucket.org/pypy/pypy/changeset/382248c1c015/
Log: Merge the string-NUL branch by amaury. Adds to the annotator the
constraint that some external functions, like open(), must be called
with strings that don't contain \x00 characters.
diff --git a/pypy/annotation/binaryop.py b/pypy/annotation/binaryop.py
--- a/pypy/annotation/binaryop.py
+++ b/pypy/annotation/binaryop.py
@@ -434,11 +434,13 @@
class __extend__(pairtype(SomeString, SomeString)):
def union((str1, str2)):
- return SomeString(can_be_None=str1.can_be_None or str2.can_be_None)
+ can_be_None = str1.can_be_None or str2.can_be_None
+ no_nul = str1.no_nul and str2.no_nul
+ return SomeString(can_be_None=can_be_None, no_nul=no_nul)
def add((str1, str2)):
# propagate const-ness to help getattr(obj, 'prefix' + const_name)
- result = SomeString()
+ result = SomeString(no_nul=str1.no_nul and str2.no_nul)
if str1.is_immutable_constant() and str2.is_immutable_constant():
result.const = str1.const + str2.const
return result
@@ -475,7 +477,16 @@
raise NotImplementedError(
"string formatting mixing strings and unicode not
supported")
getbookkeeper().count('strformat', str, s_tuple)
- return SomeString()
+ no_nul = str.no_nul
+ for s_item in s_tuple.items:
+ if isinstance(s_item, SomeFloat):
+ pass # or s_item is a subclass, like SomeInteger
+ elif isinstance(s_item, SomeString) and s_item.no_nul:
+ pass
+ else:
+ no_nul = False
+ break
+ return SomeString(no_nul=no_nul)
class __extend__(pairtype(SomeString, SomeObject)):
@@ -828,7 +839,7 @@
exec source.compile() in glob
_make_none_union('SomeInstance', 'classdef=obj.classdef, can_be_None=True')
-_make_none_union('SomeString', 'can_be_None=True')
+_make_none_union('SomeString', 'no_nul=obj.no_nul, can_be_None=True')
_make_none_union('SomeUnicodeString', 'can_be_None=True')
_make_none_union('SomeList', 'obj.listdef')
_make_none_union('SomeDict', 'obj.dictdef')
diff --git a/pypy/annotation/bookkeeper.py b/pypy/annotation/bookkeeper.py
--- a/pypy/annotation/bookkeeper.py
+++ b/pypy/annotation/bookkeeper.py
@@ -342,10 +342,11 @@
else:
raise Exception("seeing a prebuilt long (value %s)" % hex(x))
elif issubclass(tp, str): # py.lib uses annotated str subclasses
+ no_nul = not '\x00' in x
if len(x) == 1:
- result = SomeChar()
+ result = SomeChar(no_nul=no_nul)
else:
- result = SomeString()
+ result = SomeString(no_nul=no_nul)
elif tp is unicode:
if len(x) == 1:
result = SomeUnicodeCodePoint()
diff --git a/pypy/annotation/listdef.py b/pypy/annotation/listdef.py
--- a/pypy/annotation/listdef.py
+++ b/pypy/annotation/listdef.py
@@ -86,18 +86,19 @@
read_locations = self.read_locations.copy()
other_read_locations = other.read_locations.copy()
self.read_locations.update(other.read_locations)
- self.patch() # which should patch all refs to 'other'
s_value = self.s_value
s_other_value = other.s_value
s_new_value = unionof(s_value, s_other_value)
+ if s_new_value != s_value:
+ if self.dont_change_any_more:
+ raise TooLateForChange
if isdegenerated(s_new_value):
if self.bookkeeper:
self.bookkeeper.ondegenerated(self, s_new_value)
elif other.bookkeeper:
other.bookkeeper.ondegenerated(other, s_new_value)
+ self.patch() # which should patch all refs to 'other'
if s_new_value != s_value:
- if self.dont_change_any_more:
- raise TooLateForChange
self.s_value = s_new_value
# reflow from reading points
for position_key in read_locations:
@@ -222,4 +223,5 @@
MOST_GENERAL_LISTDEF = ListDef(None, SomeObject())
-s_list_of_strings = SomeList(ListDef(None, SomeString(), resized = True))
+s_list_of_strings = SomeList(ListDef(None, SomeString(no_nul=True),
+ resized = True))
diff --git a/pypy/annotation/model.py b/pypy/annotation/model.py
--- a/pypy/annotation/model.py
+++ b/pypy/annotation/model.py
@@ -229,21 +229,33 @@
"Stands for an object which is known to be a string."
knowntype = str
immutable = True
- def __init__(self, can_be_None=False):
- self.can_be_None = can_be_None
+ can_be_None=False
+ no_nul = False # No NUL character in the string.
+
+ def __init__(self, can_be_None=False, no_nul=False):
+ if can_be_None:
+ self.can_be_None = True
+ if no_nul:
+ self.no_nul = True
def can_be_none(self):
return self.can_be_None
def nonnoneify(self):
- return SomeString(can_be_None=False)
+ return SomeString(can_be_None=False, no_nul=self.no_nul)
class SomeUnicodeString(SomeObject):
"Stands for an object which is known to be an unicode string"
knowntype = unicode
immutable = True
- def __init__(self, can_be_None=False):
- self.can_be_None = can_be_None
+ can_be_None=False
+ no_nul = False
+
+ def __init__(self, can_be_None=False, no_nul=False):
+ if can_be_None:
+ self.can_be_None = True
+ if no_nul:
+ self.no_nul = True
def can_be_none(self):
return self.can_be_None
@@ -254,14 +266,16 @@
class SomeChar(SomeString):
"Stands for an object known to be a string of length 1."
can_be_None = False
- def __init__(self): # no 'can_be_None' argument here
- pass
+ def __init__(self, no_nul=False): # no 'can_be_None' argument here
+ if no_nul:
+ self.no_nul = True
class SomeUnicodeCodePoint(SomeUnicodeString):
"Stands for an object known to be a unicode codepoint."
can_be_None = False
- def __init__(self): # no 'can_be_None' argument here
- pass
+ def __init__(self, no_nul=False): # no 'can_be_None' argument here
+ if no_nul:
+ self.no_nul = True
SomeString.basestringclass = SomeString
SomeString.basecharclass = SomeChar
@@ -502,6 +516,7 @@
s_None = SomePBC([], can_be_None=True)
s_Bool = SomeBool()
s_ImpossibleValue = SomeImpossibleValue()
+s_Str0 = SomeString(no_nul=True)
# ____________________________________________________________
# weakrefs
@@ -716,8 +731,7 @@
def not_const(s_obj):
if s_obj.is_constant():
- new_s_obj = SomeObject()
- new_s_obj.__class__ = s_obj.__class__
+ new_s_obj = SomeObject.__new__(s_obj.__class__)
dic = new_s_obj.__dict__ = s_obj.__dict__.copy()
if 'const' in dic:
del new_s_obj.const
diff --git a/pypy/annotation/test/test_annrpython.py
b/pypy/annotation/test/test_annrpython.py
--- a/pypy/annotation/test/test_annrpython.py
+++ b/pypy/annotation/test/test_annrpython.py
@@ -456,6 +456,20 @@
return ''.join(g(n))
s = a.build_types(f, [int])
assert s.knowntype == str
+ assert s.no_nul
+
+ def test_str_split(self):
+ a = self.RPythonAnnotator()
+ def g(n):
+ if n:
+ return "test string"
+ def f(n):
+ if n:
+ return g(n).split(' ')
+ s = a.build_types(f, [int])
+ assert isinstance(s, annmodel.SomeList)
+ s_item = s.listdef.listitem.s_value
+ assert s_item.no_nul
def test_str_splitlines(self):
a = self.RPythonAnnotator()
@@ -1841,7 +1855,7 @@
return obj.indirect()
a = self.RPythonAnnotator()
s = a.build_types(f, [bool])
- assert s == annmodel.SomeString(can_be_None=True)
+ assert annmodel.SomeString(can_be_None=True).contains(s)
def test_dont_see_AttributeError_clause(self):
class Stuff:
@@ -2018,6 +2032,37 @@
s = a.build_types(g, [int])
assert not s.can_be_None
+ def test_string_noNUL_canbeNone(self):
+ def f(a):
+ if a:
+ return "abc"
+ else:
+ return None
+ a = self.RPythonAnnotator()
+ s = a.build_types(f, [int])
+ assert s.can_be_None
+ assert s.no_nul
+
+ def test_str_or_None(self):
+ def f(a):
+ if a:
+ return "abc"
+ else:
+ return None
+ def g(a):
+ x = f(a)
+ #assert x is not None
+ if x is None:
+ return "abcd"
+ return x
+ if isinstance(x, str):
+ return x
+ return "impossible"
+ a = self.RPythonAnnotator()
+ s = a.build_types(f, [int])
+ assert s.can_be_None
+ assert s.no_nul
+
def test_emulated_pbc_call_simple(self):
def f(a,b):
return a + b
@@ -2071,6 +2116,19 @@
assert isinstance(s, annmodel.SomeIterator)
assert s.variant == ('items',)
+ def test_iteritems_str0(self):
+ def it(d):
+ return d.iteritems()
+ def f():
+ d0 = {'1a': '2a', '3': '4'}
+ for item in it(d0):
+ return "%s=%s" % item
+ raise ValueError
+ a = self.RPythonAnnotator()
+ s = a.build_types(f, [])
+ assert isinstance(s, annmodel.SomeString)
+ assert s.no_nul
+
def test_non_none_and_none_with_isinstance(self):
class A(object):
pass
diff --git a/pypy/annotation/unaryop.py b/pypy/annotation/unaryop.py
--- a/pypy/annotation/unaryop.py
+++ b/pypy/annotation/unaryop.py
@@ -497,7 +497,8 @@
if isinstance(str, SomeUnicodeString):
return immutablevalue(u"")
return immutablevalue("")
- return str.basestringclass()
+ no_nul = str.no_nul and s_item.no_nul
+ return str.basestringclass(no_nul=no_nul)
def iter(str):
return SomeIterator(str)
@@ -508,18 +509,21 @@
def method_split(str, patt, max=-1):
getbookkeeper().count("str_split", str, patt)
- return getbookkeeper().newlist(str.basestringclass())
+ s_item = str.basestringclass(no_nul=str.no_nul)
+ return getbookkeeper().newlist(s_item)
def method_rsplit(str, patt, max=-1):
getbookkeeper().count("str_rsplit", str, patt)
- return getbookkeeper().newlist(str.basestringclass())
+ s_item = str.basestringclass(no_nul=str.no_nul)
+ return getbookkeeper().newlist(s_item)
def method_replace(str, s1, s2):
return str.basestringclass()
def getslice(str, s_start, s_stop):
check_negative_slice(s_start, s_stop)
- return str.basestringclass()
+ result = str.basestringclass(no_nul=str.no_nul)
+ return result
class __extend__(SomeUnicodeString):
def method_encode(uni, s_enc):
diff --git a/pypy/interpreter/baseobjspace.py b/pypy/interpreter/baseobjspace.py
--- a/pypy/interpreter/baseobjspace.py
+++ b/pypy/interpreter/baseobjspace.py
@@ -1312,6 +1312,15 @@
def str_w(self, w_obj):
return w_obj.str_w(self)
+ def str0_w(self, w_obj):
+ "Like str_w, but rejects strings with NUL bytes."
+ from pypy.rlib import rstring
+ result = w_obj.str_w(self)
+ if '\x00' in result:
+ raise OperationError(self.w_TypeError, self.wrap(
+ 'argument must be a string without NUL characters'))
+ return rstring.assert_str0(result)
+
def int_w(self, w_obj):
return w_obj.int_w(self)
diff --git a/pypy/interpreter/gateway.py b/pypy/interpreter/gateway.py
--- a/pypy/interpreter/gateway.py
+++ b/pypy/interpreter/gateway.py
@@ -130,6 +130,9 @@
def visit_str_or_None(self, el, app_sig):
self.checked_space_method(el, app_sig)
+ def visit_str0(self, el, app_sig):
+ self.checked_space_method(el, app_sig)
+
def visit_nonnegint(self, el, app_sig):
self.checked_space_method(el, app_sig)
@@ -249,6 +252,9 @@
def visit_str_or_None(self, typ):
self.run_args.append("space.str_or_None_w(%s)" % (self.scopenext(),))
+ def visit_str0(self, typ):
+ self.run_args.append("space.str0_w(%s)" % (self.scopenext(),))
+
def visit_nonnegint(self, typ):
self.run_args.append("space.gateway_nonnegint_w(%s)" % (
self.scopenext(),))
@@ -383,6 +389,9 @@
def visit_str_or_None(self, typ):
self.unwrap.append("space.str_or_None_w(%s)" % (self.nextarg(),))
+ def visit_str0(self, typ):
+ self.unwrap.append("space.str0_w(%s)" % (self.nextarg(),))
+
def visit_nonnegint(self, typ):
self.unwrap.append("space.gateway_nonnegint_w(%s)" % (self.nextarg(),))
diff --git a/pypy/interpreter/mixedmodule.py b/pypy/interpreter/mixedmodule.py
--- a/pypy/interpreter/mixedmodule.py
+++ b/pypy/interpreter/mixedmodule.py
@@ -50,7 +50,7 @@
space.call_method(self.w_dict, 'update', self.w_initialdict)
for w_submodule in self.submodules_w:
- name = space.str_w(w_submodule.w_name)
+ name = space.str0_w(w_submodule.w_name)
space.setitem(self.w_dict, space.wrap(name.split(".")[-1]),
w_submodule)
space.getbuiltinmodule(name)
diff --git a/pypy/interpreter/module.py b/pypy/interpreter/module.py
--- a/pypy/interpreter/module.py
+++ b/pypy/interpreter/module.py
@@ -31,7 +31,8 @@
def install(self):
"""NOT_RPYTHON: installs this module into space.builtin_modules"""
w_mod = self.space.wrap(self)
- self.space.builtin_modules[self.space.unwrap(self.w_name)] = w_mod
+ modulename = self.space.str0_w(self.w_name)
+ self.space.builtin_modules[modulename] = w_mod
def setup_after_space_initialization(self):
"""NOT_RPYTHON: to allow built-in modules to do some more setup
diff --git a/pypy/jit/metainterp/test/test_ajit.py
b/pypy/jit/metainterp/test/test_ajit.py
--- a/pypy/jit/metainterp/test/test_ajit.py
+++ b/pypy/jit/metainterp/test/test_ajit.py
@@ -322,6 +322,17 @@
res = self.interp_operations(f, [42])
assert res == ord(u"?")
+ def test_char_in_constant_string(self):
+ def g(string):
+ return '\x00' in string
+ def f():
+ if g('abcdef'): return -60
+ if not g('abc\x00ef'): return -61
+ return 42
+ res = self.interp_operations(f, [])
+ assert res == 42
+ self.check_operations_history({'finish': 1}) # nothing else
+
def test_residual_call(self):
@dont_look_inside
def externfn(x, y):
diff --git a/pypy/module/bz2/interp_bz2.py b/pypy/module/bz2/interp_bz2.py
--- a/pypy/module/bz2/interp_bz2.py
+++ b/pypy/module/bz2/interp_bz2.py
@@ -328,7 +328,7 @@
if basemode == "a":
raise OperationError(space.w_ValueError,
space.wrap("cannot append to bz2 file"))
- stream = open_path_helper(space.str_w(w_path), os_flags, False)
+ stream = open_path_helper(space.str0_w(w_path), os_flags, False)
if reading:
bz2stream = ReadBZ2Filter(space, stream, buffering)
buffering = 0 # by construction, the ReadBZ2Filter acts like
diff --git a/pypy/module/gc/interp_gc.py b/pypy/module/gc/interp_gc.py
--- a/pypy/module/gc/interp_gc.py
+++ b/pypy/module/gc/interp_gc.py
@@ -49,7 +49,7 @@
# ____________________________________________________________
-@unwrap_spec(filename=str)
+@unwrap_spec(filename='str0')
def dump_heap_stats(space, filename):
tb = rgc._heap_stats()
if not tb:
diff --git a/pypy/module/imp/importing.py b/pypy/module/imp/importing.py
--- a/pypy/module/imp/importing.py
+++ b/pypy/module/imp/importing.py
@@ -138,7 +138,7 @@
ctxt_package = None
if ctxt_w_package is not None and ctxt_w_package is not space.w_None:
try:
- ctxt_package = space.str_w(ctxt_w_package)
+ ctxt_package = space.str0_w(ctxt_w_package)
except OperationError, e:
if not e.match(space, space.w_TypeError):
raise
@@ -187,7 +187,7 @@
ctxt_name = None
if ctxt_w_name is not None:
try:
- ctxt_name = space.str_w(ctxt_w_name)
+ ctxt_name = space.str0_w(ctxt_w_name)
except OperationError, e:
if not e.match(space, space.w_TypeError):
raise
@@ -230,7 +230,7 @@
return rel_modulename, rel_level
-@unwrap_spec(name=str, level=int)
+@unwrap_spec(name='str0', level=int)
def importhook(space, name, w_globals=None,
w_locals=None, w_fromlist=None, level=-1):
modulename = name
@@ -377,8 +377,8 @@
fromlist_w = space.fixedview(w_all)
for w_name in fromlist_w:
if try_getattr(space, w_mod, w_name) is None:
- load_part(space, w_path, prefix, space.str_w(w_name),
w_mod,
- tentative=1)
+ load_part(space, w_path, prefix, space.str0_w(w_name),
+ w_mod, tentative=1)
return w_mod
else:
return first
@@ -432,7 +432,7 @@
def __init__(self, space):
pass
- @unwrap_spec(path=str)
+ @unwrap_spec(path='str0')
def descr_init(self, space, path):
if not path:
raise OperationError(space.w_ImportError, space.wrap(
@@ -513,7 +513,7 @@
if w_loader:
return FindInfo.fromLoader(w_loader)
- path = space.str_w(w_pathitem)
+ path = space.str0_w(w_pathitem)
filepart = os.path.join(path, partname)
if os.path.isdir(filepart) and case_ok(filepart):
initfile = os.path.join(filepart, '__init__')
@@ -671,7 +671,7 @@
space.wrap("reload() argument must be module"))
w_modulename = space.getattr(w_module, space.wrap("__name__"))
- modulename = space.str_w(w_modulename)
+ modulename = space.str0_w(w_modulename)
if not space.is_w(check_sys_modules(space, w_modulename), w_module):
raise operationerrfmt(
space.w_ImportError,
diff --git a/pypy/module/imp/interp_imp.py b/pypy/module/imp/interp_imp.py
--- a/pypy/module/imp/interp_imp.py
+++ b/pypy/module/imp/interp_imp.py
@@ -44,7 +44,7 @@
return space.interp_w(W_File, w_file).stream
def find_module(space, w_name, w_path=None):
- name = space.str_w(w_name)
+ name = space.str0_w(w_name)
if space.is_w(w_path, space.w_None):
w_path = None
@@ -75,7 +75,7 @@
def load_module(space, w_name, w_file, w_filename, w_info):
w_suffix, w_filemode, w_modtype = space.unpackiterable(w_info)
- filename = space.str_w(w_filename)
+ filename = space.str0_w(w_filename)
filemode = space.str_w(w_filemode)
if space.is_w(w_file, space.w_None):
stream = None
@@ -92,7 +92,7 @@
space, w_name, find_info, reuse=True)
def load_source(space, w_modulename, w_filename, w_file=None):
- filename = space.str_w(w_filename)
+ filename = space.str0_w(w_filename)
stream = get_file(space, w_file, filename, 'U')
@@ -105,7 +105,7 @@
stream.close()
return w_mod
-@unwrap_spec(filename=str)
+@unwrap_spec(filename='str0')
def _run_compiled_module(space, w_modulename, filename, w_file, w_module):
# the function 'imp._run_compiled_module' is a pypy-only extension
stream = get_file(space, w_file, filename, 'rb')
@@ -119,7 +119,7 @@
if space.is_w(w_file, space.w_None):
stream.close()
-@unwrap_spec(filename=str)
+@unwrap_spec(filename='str0')
def load_compiled(space, w_modulename, filename, w_file=None):
w_mod = space.wrap(Module(space, w_modulename))
importing._prepare_module(space, w_mod, filename, None)
@@ -138,7 +138,7 @@
return space.wrap(Module(space, w_name, add_package=False))
def init_builtin(space, w_name):
- name = space.str_w(w_name)
+ name = space.str0_w(w_name)
if name not in space.builtin_modules:
return
if space.finditem(space.sys.get('modules'), w_name) is not None:
@@ -151,7 +151,7 @@
return None
def is_builtin(space, w_name):
- name = space.str_w(w_name)
+ name = space.str0_w(w_name)
if name not in space.builtin_modules:
return space.wrap(0)
if space.finditem(space.sys.get('modules'), w_name) is not None:
diff --git a/pypy/module/posix/interp_posix.py
b/pypy/module/posix/interp_posix.py
--- a/pypy/module/posix/interp_posix.py
+++ b/pypy/module/posix/interp_posix.py
@@ -37,7 +37,7 @@
if space.isinstance_w(w_obj, space.w_unicode):
w_obj = space.call_method(w_obj, 'encode',
getfilesystemencoding(space))
- return space.str_w(w_obj)
+ return space.str0_w(w_obj)
class FileEncoder(object):
def __init__(self, space, w_obj):
@@ -56,7 +56,7 @@
self.w_obj = w_obj
def as_bytes(self):
- return self.space.str_w(self.w_obj)
+ return self.space.str0_w(self.w_obj)
def as_unicode(self):
space = self.space
@@ -71,7 +71,7 @@
fname = FileEncoder(space, w_fname)
return func(fname, *args)
else:
- fname = space.str_w(w_fname)
+ fname = space.str0_w(w_fname)
return func(fname, *args)
return dispatch
@@ -369,7 +369,7 @@
space.wrap(times[3]),
space.wrap(times[4])])
-@unwrap_spec(cmd=str)
+@unwrap_spec(cmd='str0')
def system(space, cmd):
"""Execute the command (a string) in a subshell."""
try:
@@ -401,7 +401,7 @@
fullpath = rposix._getfullpathname(path)
w_fullpath = space.wrap(fullpath)
else:
- path = space.str_w(w_path)
+ path = space.str0_w(w_path)
fullpath = rposix._getfullpathname(path)
w_fullpath = space.wrap(fullpath)
except OSError, e:
@@ -512,7 +512,7 @@
for key, value in os.environ.items():
space.setitem(w_env, space.wrap(key), space.wrap(value))
-@unwrap_spec(name=str, value=str)
+@unwrap_spec(name='str0', value='str0')
def putenv(space, name, value):
"""Change or add an environment variable."""
try:
@@ -520,7 +520,7 @@
except OSError, e:
raise wrap_oserror(space, e)
-@unwrap_spec(name=str)
+@unwrap_spec(name='str0')
def unsetenv(space, name):
"""Delete an environment variable."""
try:
@@ -548,7 +548,7 @@
for s in result
]
else:
- dirname = space.str_w(w_dirname)
+ dirname = space.str0_w(w_dirname)
result = rposix.listdir(dirname)
result_w = [space.wrap(s) for s in result]
except OSError, e:
@@ -635,7 +635,7 @@
import signal
os.kill(os.getpid(), signal.SIGABRT)
-@unwrap_spec(src=str, dst=str)
+@unwrap_spec(src='str0', dst='str0')
def link(space, src, dst):
"Create a hard link to a file."
try:
@@ -650,7 +650,7 @@
except OSError, e:
raise wrap_oserror(space, e)
-@unwrap_spec(path=str)
+@unwrap_spec(path='str0')
def readlink(space, path):
"Return a string representing the path to which the symbolic link points."
try:
@@ -765,7 +765,7 @@
w_keys = space.call_method(w_env, 'keys')
for w_key in space.unpackiterable(w_keys):
w_value = space.getitem(w_env, w_key)
- env[space.str_w(w_key)] = space.str_w(w_value)
+ env[space.str0_w(w_key)] = space.str0_w(w_value)
return env
def execve(space, w_command, w_args, w_env):
@@ -785,18 +785,18 @@
except OSError, e:
raise wrap_oserror(space, e)
-@unwrap_spec(mode=int, path=str)
+@unwrap_spec(mode=int, path='str0')
def spawnv(space, mode, path, w_args):
- args = [space.str_w(w_arg) for w_arg in space.unpackiterable(w_args)]
+ args = [space.str0_w(w_arg) for w_arg in space.unpackiterable(w_args)]
try:
ret = os.spawnv(mode, path, args)
except OSError, e:
raise wrap_oserror(space, e)
return space.wrap(ret)
-@unwrap_spec(mode=int, path=str)
+@unwrap_spec(mode=int, path='str0')
def spawnve(space, mode, path, w_args, w_env):
- args = [space.str_w(w_arg) for w_arg in space.unpackiterable(w_args)]
+ args = [space.str0_w(w_arg) for w_arg in space.unpackiterable(w_args)]
env = _env2interp(space, w_env)
try:
ret = os.spawnve(mode, path, args, env)
@@ -914,7 +914,7 @@
raise wrap_oserror(space, e)
return space.w_None
-@unwrap_spec(path=str)
+@unwrap_spec(path='str0')
def chroot(space, path):
""" chroot(path)
@@ -1103,7 +1103,7 @@
except OSError, e:
raise wrap_oserror(space, e)
-@unwrap_spec(path=str, uid=c_uid_t, gid=c_gid_t)
+@unwrap_spec(path='str0', uid=c_uid_t, gid=c_gid_t)
def chown(space, path, uid, gid):
check_uid_range(space, uid)
check_uid_range(space, gid)
@@ -1113,7 +1113,7 @@
raise wrap_oserror(space, e, path)
return space.w_None
-@unwrap_spec(path=str, uid=c_uid_t, gid=c_gid_t)
+@unwrap_spec(path='str0', uid=c_uid_t, gid=c_gid_t)
def lchown(space, path, uid, gid):
check_uid_range(space, uid)
check_uid_range(space, gid)
diff --git a/pypy/module/sys/state.py b/pypy/module/sys/state.py
--- a/pypy/module/sys/state.py
+++ b/pypy/module/sys/state.py
@@ -74,7 +74,7 @@
#
return importlist
-@unwrap_spec(srcdir=str)
+@unwrap_spec(srcdir='str0')
def pypy_initial_path(space, srcdir):
try:
path = getinitialpath(get(space), srcdir)
diff --git a/pypy/module/zipimport/interp_zipimport.py
b/pypy/module/zipimport/interp_zipimport.py
--- a/pypy/module/zipimport/interp_zipimport.py
+++ b/pypy/module/zipimport/interp_zipimport.py
@@ -342,7 +342,7 @@
space = self.space
return space.wrap(self.filename)
-@unwrap_spec(name=str)
+@unwrap_spec(name='str0')
def descr_new_zipimporter(space, w_type, name):
w = space.wrap
ok = False
diff --git a/pypy/rlib/rstring.py b/pypy/rlib/rstring.py
--- a/pypy/rlib/rstring.py
+++ b/pypy/rlib/rstring.py
@@ -205,3 +205,45 @@
assert p.const is None
return SomeUnicodeBuilder(can_be_None=True)
+#___________________________________________________________________
+# Support functions for SomeString.no_nul
+
+def assert_str0(fname):
+ assert '\x00' not in fname, "NUL byte in string"
+ return fname
+
+class Entry(ExtRegistryEntry):
+ _about_ = assert_str0
+
+ def compute_result_annotation(self, s_obj):
+ if s_None.contains(s_obj):
+ return s_obj
+ assert isinstance(s_obj, (SomeString, SomeUnicodeString))
+ if s_obj.no_nul:
+ return s_obj
+ new_s_obj = SomeObject.__new__(s_obj.__class__)
+ new_s_obj.__dict__ = s_obj.__dict__.copy()
+ new_s_obj.no_nul = True
+ return new_s_obj
+
+ def specialize_call(self, hop):
+ hop.exception_cannot_occur()
+ return hop.inputarg(hop.args_r[0], arg=0)
+
+def check_str0(fname):
+ """A 'probe' to trigger a failure at translation time, if the
+ string was not proved to not contain NUL characters."""
+ assert '\x00' not in fname, "NUL byte in string"
+
+class Entry(ExtRegistryEntry):
+ _about_ = check_str0
+
+ def compute_result_annotation(self, s_obj):
+ if not isinstance(s_obj, (SomeString, SomeUnicodeString)):
+ return s_obj
+ if not s_obj.no_nul:
+ raise ValueError("Value is not no_nul")
+
+ def specialize_call(self, hop):
+ pass
+
diff --git a/pypy/rlib/test/test_rmarshal.py b/pypy/rlib/test/test_rmarshal.py
--- a/pypy/rlib/test/test_rmarshal.py
+++ b/pypy/rlib/test/test_rmarshal.py
@@ -169,7 +169,7 @@
assert st2.st_mode == st.st_mode
assert st2[9] == st[9]
return buf
- fn = compile(f, [str])
+ fn = compile(f, [annmodel.s_Str0])
res = fn('.')
st = os.stat('.')
sttuple = marshal.loads(res)
diff --git a/pypy/rpython/extfuncregistry.py b/pypy/rpython/extfuncregistry.py
--- a/pypy/rpython/extfuncregistry.py
+++ b/pypy/rpython/extfuncregistry.py
@@ -85,7 +85,8 @@
# llinterpreter
path_functions = [
- ('join', [str, str], str),
+ ('join', [ll_os.str0, ll_os.str0], ll_os.str0),
+ ('dirname', [ll_os.str0], ll_os.str0),
]
for name, args, res in path_functions:
diff --git a/pypy/rpython/lltypesystem/rffi.py
b/pypy/rpython/lltypesystem/rffi.py
--- a/pypy/rpython/lltypesystem/rffi.py
+++ b/pypy/rpython/lltypesystem/rffi.py
@@ -15,7 +15,7 @@
from pypy.translator.tool.cbuild import ExternalCompilationInfo
from pypy.rpython.annlowlevel import llhelper
from pypy.rlib.objectmodel import we_are_translated
-from pypy.rlib.rstring import StringBuilder, UnicodeBuilder
+from pypy.rlib.rstring import StringBuilder, UnicodeBuilder, assert_str0
from pypy.rlib import jit
from pypy.rpython.lltypesystem import llmemory
import os, sys
@@ -698,7 +698,7 @@
while cp[i] != lastchar:
b.append(cp[i])
i += 1
- return b.build()
+ return assert_str0(b.build())
# str -> char*
# Can't inline this because of the raw address manipulation.
@@ -804,7 +804,7 @@
while i < maxlen and cp[i] != lastchar:
b.append(cp[i])
i += 1
- return b.build()
+ return assert_str0(b.build())
# char* and size -> str (which can contain null bytes)
def charpsize2str(cp, size):
@@ -842,6 +842,7 @@
array[i] = str2charp(l[i])
array[len(l)] = lltype.nullptr(CCHARP.TO)
return array
+liststr2charpp._annenforceargs_ = [[annmodel.s_Str0]] # List of strings
def free_charpp(ref):
""" frees list of char**, NULL terminated
diff --git a/pypy/rpython/module/ll_os_environ.py
b/pypy/rpython/module/ll_os_environ.py
--- a/pypy/rpython/module/ll_os_environ.py
+++ b/pypy/rpython/module/ll_os_environ.py
@@ -5,6 +5,8 @@
from pypy.rpython.lltypesystem import rffi, lltype
from pypy.rlib import rposix
+str0 = annmodel.s_Str0
+
# ____________________________________________________________
#
# Annotation support to control access to 'os.environ' in the RPython program
@@ -64,7 +66,7 @@
rffi.free_charp(l_name)
return result
-register_external(r_getenv, [str], annmodel.SomeString(can_be_None=True),
+register_external(r_getenv, [str0], annmodel.SomeString(can_be_None=True),
export_name='ll_os.ll_os_getenv',
llimpl=getenv_llimpl)
@@ -93,7 +95,7 @@
if l_oldstring:
rffi.free_charp(l_oldstring)
-register_external(r_putenv, [str, str], annmodel.s_None,
+register_external(r_putenv, [str0, str0], annmodel.s_None,
export_name='ll_os.ll_os_putenv',
llimpl=putenv_llimpl)
@@ -128,7 +130,7 @@
del envkeepalive.byname[name]
rffi.free_charp(l_oldstring)
- register_external(r_unsetenv, [str], annmodel.s_None,
+ register_external(r_unsetenv, [str0], annmodel.s_None,
export_name='ll_os.ll_os_unsetenv',
llimpl=unsetenv_llimpl)
@@ -172,7 +174,7 @@
i += 1
return result
-register_external(r_envkeys, [], [str], # returns a list of strings
+register_external(r_envkeys, [], [str0], # returns a list of strings
export_name='ll_os.ll_os_envkeys',
llimpl=envkeys_llimpl)
@@ -193,6 +195,6 @@
i += 1
return result
-register_external(r_envitems, [], [(str, str)],
+register_external(r_envitems, [], [(str0, str0)],
export_name='ll_os.ll_os_envitems',
llimpl=envitems_llimpl)
diff --git a/pypy/rpython/module/ll_os_stat.py
b/pypy/rpython/module/ll_os_stat.py
--- a/pypy/rpython/module/ll_os_stat.py
+++ b/pypy/rpython/module/ll_os_stat.py
@@ -236,7 +236,7 @@
def register_stat_variant(name, traits):
if name != 'fstat':
arg_is_path = True
- s_arg = traits.str
+ s_arg = traits.str0
ARG1 = traits.CCHARP
else:
arg_is_path = False
@@ -251,8 +251,6 @@
[s_arg], s_StatResult, traits.ll_os_name(name),
llimpl=posix_stat_llimpl)
- assert traits.str is str
-
if sys.platform.startswith('linux'):
# because we always use _FILE_OFFSET_BITS 64 - this helps things work
that are not a c compiler
_functions = {'stat': 'stat64',
@@ -283,7 +281,7 @@
@func_renamer('os_%s_fake' % (name,))
def posix_fakeimpl(arg):
- if s_arg == str:
+ if s_arg == traits.str0:
arg = hlstr(arg)
st = getattr(os, name)(arg)
fields = [TYPE for fieldname, TYPE in STAT_FIELDS]
diff --git a/pypy/rpython/ootypesystem/test/test_ooann.py
b/pypy/rpython/ootypesystem/test/test_ooann.py
--- a/pypy/rpython/ootypesystem/test/test_ooann.py
+++ b/pypy/rpython/ootypesystem/test/test_ooann.py
@@ -231,7 +231,7 @@
a = RPythonAnnotator()
s = a.build_types(oof, [bool])
- assert s == annmodel.SomeString(can_be_None=True)
+ assert annmodel.SomeString(can_be_None=True).contains(s)
def test_oostring():
def oof():
diff --git a/pypy/translator/c/test/test_extfunc.py
b/pypy/translator/c/test/test_extfunc.py
--- a/pypy/translator/c/test/test_extfunc.py
+++ b/pypy/translator/c/test/test_extfunc.py
@@ -3,6 +3,7 @@
import os, time, sys
from pypy.tool.udir import udir
from pypy.rlib.rarithmetic import r_longlong
+from pypy.annotation import model as annmodel
from pypy.translator.c.test.test_genc import compile
from pypy.translator.c.test.test_standalone import StandaloneTests
posix = __import__(os.name)
@@ -145,7 +146,7 @@
filename = str(py.path.local(__file__))
def call_access(path, mode):
return os.access(path, mode)
- f = compile(call_access, [str, int])
+ f = compile(call_access, [annmodel.s_Str0, int])
for mode in os.R_OK, os.W_OK, os.X_OK, (os.R_OK | os.W_OK | os.X_OK):
assert f(filename, mode) == os.access(filename, mode)
@@ -225,7 +226,7 @@
def test_system():
def does_stuff(cmd):
return os.system(cmd)
- f1 = compile(does_stuff, [str])
+ f1 = compile(does_stuff, [annmodel.s_Str0])
res = f1("echo hello")
assert res == 0
@@ -311,7 +312,7 @@
def test_chdir():
def does_stuff(path):
os.chdir(path)
- f1 = compile(does_stuff, [str])
+ f1 = compile(does_stuff, [annmodel.s_Str0])
curdir = os.getcwd()
try:
os.chdir('..')
@@ -325,7 +326,7 @@
os.rmdir(path)
else:
os.mkdir(path, 0777)
- f1 = compile(does_stuff, [str, bool])
+ f1 = compile(does_stuff, [annmodel.s_Str0, bool])
dirname = str(udir.join('test_mkdir_rmdir'))
f1(dirname, False)
assert os.path.exists(dirname) and os.path.isdir(dirname)
@@ -628,7 +629,7 @@
return os.environ[s]
except KeyError:
return '--missing--'
- func = compile(fn, [str])
+ func = compile(fn, [annmodel.s_Str0])
os.environ.setdefault('USER', 'UNNAMED_USER')
result = func('USER')
assert result == os.environ['USER']
@@ -640,7 +641,7 @@
res = os.environ.get(s)
if res is None: res = '--missing--'
return res
- func = compile(fn, [str])
+ func = compile(fn, [annmodel.s_Str0])
os.environ.setdefault('USER', 'UNNAMED_USER')
result = func('USER')
assert result == os.environ['USER']
@@ -654,7 +655,7 @@
os.environ[s] = t3
os.environ[s] = t4
os.environ[s] = t5
- func = compile(fn, [str, str, str, str, str, str])
+ func = compile(fn, [annmodel.s_Str0] * 6)
func('PYPY_TEST_DICTLIKE_ENVIRON', 'a', 'b', 'c', 'FOOBAR', '42',
expected_extra_mallocs = (2, 3, 4)) # at least two, less than 5
assert _real_getenv('PYPY_TEST_DICTLIKE_ENVIRON') == '42'
@@ -678,7 +679,7 @@
else:
raise Exception("should have raised!")
# os.environ[s5] stays
- func = compile(fn, [str, str, str, str, str])
+ func = compile(fn, [annmodel.s_Str0] * 5)
if hasattr(__import__(os.name), 'unsetenv'):
expected_extra_mallocs = range(2, 10)
# at least 2, less than 10: memory for s1, s2, s3, s4 should be freed
@@ -743,7 +744,7 @@
raise AssertionError("should have failed!")
result = os.listdir(s)
return '/'.join(result)
- func = compile(mylistdir, [str])
+ func = compile(mylistdir, [annmodel.s_Str0])
for testdir in [str(udir), os.curdir]:
result = func(testdir)
result = result.split('/')
diff --git a/pypy/translator/cli/test/runtest.py
b/pypy/translator/cli/test/runtest.py
--- a/pypy/translator/cli/test/runtest.py
+++ b/pypy/translator/cli/test/runtest.py
@@ -276,7 +276,7 @@
def get_annotation(x):
if isinstance(x, basestring) and len(x) > 1:
- return SomeString()
+ return SomeString(no_nul='\x00' not in x)
else:
return lltype_to_annotation(typeOf(x))
diff --git a/pypy/translator/driver.py b/pypy/translator/driver.py
--- a/pypy/translator/driver.py
+++ b/pypy/translator/driver.py
@@ -184,6 +184,7 @@
self.standalone = standalone
if standalone:
+ # the 'argv' parameter
inputtypes = [s_list_of_strings]
self.inputtypes = inputtypes
diff --git a/pypy/translator/goal/nanos.py b/pypy/translator/goal/nanos.py
--- a/pypy/translator/goal/nanos.py
+++ b/pypy/translator/goal/nanos.py
@@ -266,7 +266,7 @@
raise NotImplementedError("os.name == %r" % (os.name,))
def getenv(space, w_name):
- name = space.str_w(w_name)
+ name = space.str0_w(w_name)
return space.wrap(os.environ.get(name))
getenv_w = interp2app(getenv)
_______________________________________________
pypy-commit mailing list
[email protected]
http://mail.python.org/mailman/listinfo/pypy-commit
diff --git a/pypy/rpython/module/ll_os.py b/pypy/rpython/module/ll_os.py
--- a/pypy/rpython/module/ll_os.py
+++ b/pypy/rpython/module/ll_os.py
@@ -31,6 +31,10 @@
from pypy.rlib import rgc
from pypy.rlib.objectmodel import specialize
+str0 = SomeString(no_nul=True)
+unicode0 = SomeUnicodeString(no_nul=True)
+
+
def monkeypatch_rposix(posixfunc, unicodefunc, signature):
func_name = posixfunc.__name__
@@ -68,6 +72,7 @@
class StringTraits:
str = str
+ str0 = str0
CHAR = rffi.CHAR
CCHARP = rffi.CCHARP
charp2str = staticmethod(rffi.charp2str)
@@ -85,6 +90,7 @@
class UnicodeTraits:
str = unicode
+ str0 = unicode0
CHAR = rffi.WCHAR_T
CCHARP = rffi.CWCHARP
charp2str = staticmethod(rffi.wcharp2unicode)
@@ -301,7 +307,7 @@
rffi.free_charpp(l_args)
raise OSError(rposix.get_errno(), "execv failed")
- return extdef([str, [str]], s_ImpossibleValue, llimpl=execv_llimpl,
+ return extdef([str0, [str0]], s_ImpossibleValue, llimpl=execv_llimpl,
export_name="ll_os.ll_os_execv")
@@ -319,7 +325,8 @@
# appropriate
envstrs = []
for item in env.iteritems():
- envstrs.append("%s=%s" % item)
+ envstr = "%s=%s" % item
+ envstrs.append(envstr)
l_args = rffi.liststr2charpp(args)
l_env = rffi.liststr2charpp(envstrs)
@@ -332,7 +339,7 @@
raise OSError(rposix.get_errno(), "execve failed")
return extdef(
- [str, [str], {str: str}],
+ [str0, [str0], {str0: str0}],
s_ImpossibleValue,
llimpl=execve_llimpl,
export_name="ll_os.ll_os_execve")
@@ -353,7 +360,7 @@
raise OSError(rposix.get_errno(), "os_spawnv failed")
return rffi.cast(lltype.Signed, childpid)
- return extdef([int, str, [str]], int, llimpl=spawnv_llimpl,
+ return extdef([int, str0, [str0]], int, llimpl=spawnv_llimpl,
export_name="ll_os.ll_os_spawnv")
@registering_if(os, 'spawnve')
@@ -378,7 +385,7 @@
raise OSError(rposix.get_errno(), "os_spawnve failed")
return rffi.cast(lltype.Signed, childpid)
- return extdef([int, str, [str], {str: str}], int,
+ return extdef([int, str0, [str0], {str0: str0}], int,
llimpl=spawnve_llimpl,
export_name="ll_os.ll_os_spawnve")
@@ -517,7 +524,7 @@
else:
raise Exception("os.utime() arg 2 must be None or a tuple of "
"2 floats, got %s" % (s_times,))
- os_utime_normalize_args._default_signature_ = [traits.str, None]
+ os_utime_normalize_args._default_signature_ = [traits.str0, None]
return extdef(os_utime_normalize_args, s_None,
"ll_os.ll_os_utime",
@@ -612,7 +619,7 @@
if result == -1:
raise OSError(rposix.get_errno(), "os_chroot failed")
- return extdef([str], None, export_name="ll_os.ll_os_chroot",
+ return extdef([str0], None, export_name="ll_os.ll_os_chroot",
llimpl=chroot_llimpl)
@registering_if(os, 'uname')
@@ -816,7 +823,7 @@
def os_open_oofakeimpl(path, flags, mode):
return os.open(OOSupport.from_rstr(path), flags, mode)
- return extdef([traits.str, int, int], int, traits.ll_os_name('open'),
+ return extdef([str0, int, int], int, traits.ll_os_name('open'),
llimpl=os_open_llimpl, oofakeimpl=os_open_oofakeimpl)
@registering_if(os, 'getloadavg')
@@ -1050,7 +1057,7 @@
def os_access_oofakeimpl(path, mode):
return os.access(OOSupport.from_rstr(path), mode)
- return extdef([traits.str, int], s_Bool, llimpl=access_llimpl,
+ return extdef([traits.str0, int], s_Bool, llimpl=access_llimpl,
export_name=traits.ll_os_name("access"),
oofakeimpl=os_access_oofakeimpl)
@@ -1062,8 +1069,8 @@
from pypy.rpython.module.ll_win32file import make_getfullpathname_impl
getfullpathname_llimpl = make_getfullpathname_impl(traits)
- return extdef([traits.str], # a single argument which is a str
- traits.str, # returns a string
+ return extdef([traits.str0], # a single argument which is a str
+ traits.str0, # returns a string
traits.ll_os_name('_getfullpathname'),
llimpl=getfullpathname_llimpl)
@@ -1174,8 +1181,8 @@
raise OSError(error, "os_readdir failed")
return result
- return extdef([traits.str], # a single argument which is a str
- [traits.str], # returns a list of strings
+ return extdef([traits.str0], # a single argument which is a str
+ [traits.str0], # returns a list of strings
traits.ll_os_name('listdir'),
llimpl=os_listdir_llimpl)
@@ -1241,7 +1248,7 @@
if res == -1:
raise OSError(rposix.get_errno(), "os_chown failed")
- return extdef([str, int, int], None, "ll_os.ll_os_chown",
+ return extdef([str0, int, int], None, "ll_os.ll_os_chown",
llimpl=os_chown_llimpl)
@registering_if(os, 'lchown')
@@ -1254,7 +1261,7 @@
if res == -1:
raise OSError(rposix.get_errno(), "os_lchown failed")
- return extdef([str, int, int], None, "ll_os.ll_os_lchown",
+ return extdef([str0, int, int], None, "ll_os.ll_os_lchown",
llimpl=os_lchown_llimpl)
@registering_if(os, 'readlink')
@@ -1283,12 +1290,11 @@
lltype.free(buf, flavor='raw')
bufsize *= 4
# convert the result to a string
- l = [buf[i] for i in range(res)]
- result = ''.join(l)
+ result = rffi.charp2strn(buf, res)
lltype.free(buf, flavor='raw')
return result
- return extdef([str], str,
+ return extdef([str0], str0,
"ll_os.ll_os_readlink",
llimpl=os_readlink_llimpl)
@@ -1361,7 +1367,7 @@
res = os_system(command)
return rffi.cast(lltype.Signed, res)
- return extdef([str], int, llimpl=system_llimpl,
+ return extdef([str0], int, llimpl=system_llimpl,
export_name="ll_os.ll_os_system")
@registering_str_unicode(os.unlink)
@@ -1383,7 +1389,7 @@
if not win32traits.DeleteFile(path):
raise rwin32.lastWindowsError()
- return extdef([traits.str], s_None, llimpl=unlink_llimpl,
+ return extdef([traits.str0], s_None, llimpl=unlink_llimpl,
export_name=traits.ll_os_name('unlink'))
@registering_str_unicode(os.chdir)
@@ -1401,7 +1407,7 @@
from pypy.rpython.module.ll_win32file import make_chdir_impl
os_chdir_llimpl = make_chdir_impl(traits)
- return extdef([traits.str], s_None, llimpl=os_chdir_llimpl,
+ return extdef([traits.str0], s_None, llimpl=os_chdir_llimpl,
export_name=traits.ll_os_name('chdir'))
@registering_str_unicode(os.mkdir)
@@ -1424,7 +1430,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_mkdir failed")
- return extdef([traits.str, int], s_None, llimpl=os_mkdir_llimpl,
+ return extdef([traits.str0, int], s_None, llimpl=os_mkdir_llimpl,
export_name=traits.ll_os_name('mkdir'))
@registering_str_unicode(os.rmdir)
@@ -1437,7 +1443,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_rmdir failed")
- return extdef([traits.str], s_None, llimpl=rmdir_llimpl,
+ return extdef([traits.str0], s_None, llimpl=rmdir_llimpl,
export_name=traits.ll_os_name('rmdir'))
@registering_str_unicode(os.chmod)
@@ -1454,7 +1460,7 @@
from pypy.rpython.module.ll_win32file import make_chmod_impl
chmod_llimpl = make_chmod_impl(traits)
- return extdef([traits.str, int], s_None, llimpl=chmod_llimpl,
+ return extdef([traits.str0, int], s_None, llimpl=chmod_llimpl,
export_name=traits.ll_os_name('chmod'))
@registering_str_unicode(os.rename)
@@ -1476,7 +1482,7 @@
if not win32traits.MoveFile(oldpath, newpath):
raise rwin32.lastWindowsError()
- return extdef([traits.str, traits.str], s_None, llimpl=rename_llimpl,
+ return extdef([traits.str0, traits.str0], s_None, llimpl=rename_llimpl,
export_name=traits.ll_os_name('rename'))
@registering_str_unicode(getattr(os, 'mkfifo', None))
@@ -1489,7 +1495,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_mkfifo failed")
- return extdef([traits.str, int], s_None, llimpl=mkfifo_llimpl,
+ return extdef([traits.str0, int], s_None, llimpl=mkfifo_llimpl,
export_name=traits.ll_os_name('mkfifo'))
@registering_str_unicode(getattr(os, 'mknod', None))
@@ -1503,7 +1509,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_mknod failed")
- return extdef([traits.str, int, int], s_None, llimpl=mknod_llimpl,
+ return extdef([traits.str0, int, int], s_None, llimpl=mknod_llimpl,
export_name=traits.ll_os_name('mknod'))
@registering(os.umask)
@@ -1555,7 +1561,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_link failed")
- return extdef([str, str], s_None, llimpl=link_llimpl,
+ return extdef([str0, str0], s_None, llimpl=link_llimpl,
export_name="ll_os.ll_os_link")
@registering_if(os, 'symlink')
@@ -1568,7 +1574,7 @@
if res < 0:
raise OSError(rposix.get_errno(), "os_symlink failed")
- return extdef([str, str], s_None, llimpl=symlink_llimpl,
+ return extdef([str0, str0], s_None, llimpl=symlink_llimpl,
export_name="ll_os.ll_os_symlink")
@registering_if(os, 'fork')